package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.Material
import dev.fyloz.colorrecipesexplorer.model.MaterialType
import dev.fyloz.colorrecipesexplorer.model.MixType
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MixTypeRepository : NamedJpaRepository<MixType> {
    @Query("select case when(count(m) > 0) then true else false end from MixType m where m.name = :name and m.material.materialType = :materialType")
    fun existsByNameAndMaterialType(name: String, materialType: MaterialType): Boolean

    /** Gets the mix type with the given [material]. */
    fun findByMaterial(material: Material): MixType?

    /** Gets the [MixType] with the given [name] and [materialType]. */
    @Query("select m from MixType m where m.name = :name and m.material.materialType = :materialType")
    fun findByNameAndMaterialType(name: String, materialType: MaterialType): MixType?

    @Query(
      """
        select case when(count(m.id) > 0) then false else true end
        from MixType t
        left join Mix m on t.id = m.mixType.id
        where t.id = :id
    """
    )
    fun canBeDeleted(id: Long): Boolean
}
